/** base class for conditional exception handling.
  */
public abstract class Activation extends Throwable {
  /** describes who called this activation.
    */
  private final Activation caller;

  /** creates an activation.
      @param caller activation corresponding to method who called us.
    */
  protected Activation (Activation caller) { this.caller = caller; }

  /** (conditionally) sends an object up the activation chain.
      This method should be overwritten in a concrete class.  It is called
      by a descendant to try to send an object back.  It can interact
      with the creator of the activation to decide if the object should
      be handled by the creator.  If so, it executes "handle(object)";
      if not, it executes "super.raise(object)" to send the object
      further up or "return object" to force the descendant down the
      activation chain to deal with the object.
      @param info to be sent up the chain.
      @throws Activation to be caught by the creator who should retrieve the 	object by calling info().
      @throws NullPointerException if this is the first activation in a chain.
      @return info back to descendant.
    */
  public Object raise (Object info) throws Activation {
    return caller.raise(info);
  }

  /** sends an object to the creator.
      The creator should be set up as follows: <pre>
	Activation frame = new MyActivation(myCaller);
	try {
		// something
	} catch (Activation a) {
	  if (a != frame) throw a;
	  Object info = a.info();
		// deal with info
	}
</pre>
    */
  protected final void handle (Object info) throws Activation {
    this.info = info;
    throw this;
  }

  /** references object for retrieval by creator.
    */
  private Object info;

  /** @return info.
    */
  public final Object info () { return info; }
}
